********************************************************************************
*This file predicts the CBDC demand under different designs and the percentage changes in CBDC demand in response to changes in attributes. 
*Author: Jiaqi Li


macro drop _all


global date 		= "29Oct21"
global workdir 		= "B:\CBDC-Li"
global data 		= "$workdir\data"
global code 		= "$workdir\code\Submission"
global results 		= "$workdir\results\\$date"


clear all
set maxvar 120000


cap mkdir "$data\\counterfactual analysis\\$date"
cap mkdir "$data\\counterfactual analysis\\$date\\no bank responses"

********************************************************************************
*Keep the variables that will be used in counterfactual analysis 
********************************************************************************

clear
use "$data/Data_varlabel_edited_v4_MOP13.dta"

gen anonymI = -(anonym - 1)
gen budgetI = -(budget - 1)
gen cons = 1

rename regioncat5   regioncat
rename ageheadcat5  ageheadcat
rename att_risk1    risk
rename att_debt1    debt

//estimate demand parameters  
local attributelist  rate  bundling  reldif_costopp_dc  reldif_ease_dc  reldif_secur_dc  anonymI  budgetI  sn_loc_online_mop  frac_card_unaccep
local demolist1    risk   debt  ind_behinddebt  ind_rent  ind_inetwork   female  ind_rural   ind_TD  ind_RBC  ind_LAU  ind_OT 
local demolist2    ageheadcat  ind_hldincome  hldsizecat  educhead  year  regioncat 
local demolist2i   i.ageheadcat  i.ind_hldincome  i.hldsizecat  i.educhead   i.year  i.regioncat 
reg  log_dep3_cash  `attributelist'  `demolist1'  `demolist2i'  if inrange(year, 2010, 2017),  vce(robust)

//save fitted value 
cap drop sample
cap drop xb
qui gen sample = 1 if e(sample)
predict xb if sample == 1 , xb

//generate wealth weight 
qui gen wealth = cash + tot_dep3 if sample == 1       
winsor wealth, gen(wealth_w) p(0.01) highonly
bys year: egen tot_wealth_w = sum(wealth_w)          
qui gen wealth_frac = wealth_w/tot_wealth_w          

//generate predicted cash and deposit shares before introducing CBDC
qui gen s_cash_old = 1/(1 + exp(xb)) if sample == 1                      
qui gen s_dep_old = 1 - s_cash_old if sample == 1						
	
//keep selected variables to reduce data size 
keep id hldid year log_dep3_cash  `attributelist'  `demolist1'  `demolist2' cons  sample wealth wealth_w wealth_frac s_cash_old s_dep_old  xb  costopp_*  ease_*  secur_*  sum_*

save "$data//Data_varlabel_edited_v4_MOP13_selectedvars.dta", replace


********************************************************************************
*Program for counterfactual analysis 
********************************************************************************

cap program drop prediction 
program define prediction

	args myinput 
	//job is defined as `model'XXX`design'XXX`charname'XXX`rho'XXX`gamma'XXX`eta'
	local myinput    = subinstr("`myinput'","XXX"," ",.)
	tokenize "`myinput'"
	local model  	 = "`1'"
	local design     = "`2'"
	local charname   = "`3'"
	local rho    	 = "`4'"
	local gamma      = "`5'"
	local eta        = "`6'"	

	local tau        = sqrt(1 - `rho')               /*Under baseline logit model, tau = 1*/
	di " model = `model' ;    design = `design'  ;  charname = `charname' ;   rho = `rho'  (tau = `tau') ;   gamma = `gamma' ;   eta = `eta' ;   "

	*---------------------------------------------------------------------------
	*OBTAIN DEMAND PARAMETERS
	*---------------------------------------------------------------------------
	clear
	use  "$data//Data_varlabel_edited_v4_MOP13_selectedvars.dta", clear
	
	//estimate demand parameters  
	local attributelist  rate  bundling  reldif_costopp_dc  reldif_ease_dc  reldif_secur_dc  anonymI  budgetI  sn_loc_online_mop  frac_card_unaccep
	local demolist1   risk   debt  ind_behinddebt  ind_rent  ind_inetwork   female  ind_rural   ind_TD   ind_RBC  ind_LAU  ind_OT 
	local demolist2   ageheadcat  ind_hldincome  hldsizecat  educhead  year  regioncat 
	local demolist2i   i.ageheadcat  i.ind_hldincome  i.hldsizecat  i.educhead   i.year  i.regioncat 
	qui  reg  log_dep3_cash `attributelist'  `demolist1'  `demolist2i'  if inrange(year, 2010, 2017),  vce(robust)
	
	//save covariance matrix to compute confidence intervals later
	mat V = e(V)
	
	//store coefficients and create dummies 
	foreach v of local attributelist {
		scalar b_`v' = _b[`v']
	}
	foreach v of local demolist1 {
		scalar b_`v' = _b[`v']
	}
	foreach v of local demolist2 {
		//count categories 
		qui sum `v'   if  e(sample)
		local min = r(min)
		local max = r(max)
		
		forval ii = `min'(1)`max' {
			cap drop `v'`ii'
			gen `v'`ii' = (`v' == `ii')                  
			replace `v'`ii' = . if `v' == .              /*replace missing!*/
			//`max' > `min' indicates there is at least one category
			if `ii' == `min' & `max' > `min'  {
				scalar b_`v'`ii' = _b[`ii'b.`v']        /*base category*/
			}
			else {
				scalar b_`v'`ii' = _b[`ii'.`v']
			}
		}
	}
	scalar b_cons = _b[_cons]
	
	//local categorical variables used in demolist and xlist below
	foreach v of local demolist2 {
		qui sum `v'  if  e(sample)
		local min = r(min)
		local max = r(max)
		
		local `v'_list
		forval ii = `min'(1)`max' {
			local `v'_list ``v'_list' `v'`ii'
		}
	}
	//demographics list - parameters  gamma
	local demolist  `demolist1'  `ageheadcat_list'  `ind_hldincome_list'  `hldsizecat_list'  `educhead_list'  `year_list'  `regioncat_list' 
	//!!NOTE THAT xlist' NEEDS TO BE IN THE SAME ORDER OF MATRIX V
	local xlist   `attributelist'  `demolist1'   `ageheadcat_list'  `ind_hldincome_list'  `hldsizecat_list'  `educhead_list'  `year_list'  `regioncat_list' cons
	
	*---------------------------------------------------------------------------
	*SET CBDC DESIGNS AND ATTRIBUTES TO BE LOOPED OVER
	*---------------------------------------------------------------------------
	//designs
	mat mat_d    = 999             /*deposit design*/
	mat mat_c    = 222             /*cash design*/
	mat mat_real = 555             /*baseline design*/
	//attributes - each feature matrix corresponds to a product attribute				
	mat mat_rate     		= 0, 0.02, 0.025, 0.04, 0.05, 0.06, 0.075, 0.08, 0.1					/*rate of return*/
	mat mat_bundling 		= 0, 0.2, 0.4, 0.6, 0.8, 1                         /*bundling of financial advice*/
	mat mat_online   		= 0, 0.2, 0.4, 0.6, 0.8, 1                         /*online*/
	mat mat_anonym  		= 0, 0.2, 0.4, 0.6, 0.8, 1                         /*anonymity*/
	mat mat_budget   		= 0, 0.2, 0.4, 0.6, 0.8, 1                         /*budgeting*/
	mat mat_cost_ease_secur = 1, 2, 3, 4, 5                           	/*cost_ease_secur (i.e. change all three together)  1-cc 2-ca 3-dc 4-pc 5-mo  */
	mat mat_cost	        = 1, 2, 3, 4, 5
	mat mat_ease	        = 1, 2, 3, 4, 5
	mat mat_secur           = 1, 2, 3, 4, 5
	mat mat_unaccep         = 0, 0.05, 0.1, 0.15, 0.2, 0.25                    /*card unacceptance*/

	*---------------------------------------------------------------------------
	*CREATE MATRICES TO SAVE RESULTS LATER
	*---------------------------------------------------------------------------

	//char_num = 2 if rate etc, otherwise char_num = 1
	if "`charname'" == "d" | "`charname'" == "c" | "`charname'" == "real" {
		local char_num = 1
		local row_num = 1
		local attr_list attr_`charname'
	}
	else {
		local char_num = 2
		local row_num = colsof(mat_`charname') + 1        /*first row to save the baseline result, the other rows to save the results for the changes in attributes*/
		local attr_list baseline attr_`charname'
	}
	
	//save data file for each year
	local yrmin = 2010
	local yrmax = 2017
	//column names of the matrix
	local columnvars rho gamma eta s_cash_mean s_dep_mean s_CBDC_mean as_cash as_dep as_CBDC  ///
		dl_cash  dl_dep  dl_CBDC dlp_cash  dlp_dep  dlp_CBDC d_cash_mean d_cash_median d_dep_mean d_dep_median     ///
		as_cash_lo as_cash_hi as_dep_lo as_dep_hi as_CBDC_lo as_CBDC_hi  dl_cash_lo dl_cash_hi  dl_dep_lo dl_dep_hi  dl_CBDC_lo dl_CBDC_hi   ///
		dlp_cash_lo dlp_cash_hi  dlp_dep_lo dlp_dep_hi  dlp_CBDC_lo dlp_CBDC_hi   ///
		d_cash_mean_lo d_cash_mean_hi d_cash_median_lo d_cash_median_hi d_dep_mean_lo d_dep_mean_hi d_dep_median_lo d_dep_median_hi   
	
	local columnvars_num : word count `columnvars'

	forval yr = `yrmin'(1)`yrmax' {                    
			/*create a matrix for each year*/
			mat m_s_assets_`yr' = J(`row_num', `= `char_num' + `columnvars_num'', .)            
			mat colnames m_s_assets_`yr' =   `attr_list'  `columnvars'
	}
	
	*---------------------------------------------------------------------------
	*DESIGN SCENARIOS AND CHANGES IN ATTRIBUTES
	*---------------------------------------------------------------------------	
	
	//generate temporary variables
	foreach ii in rate cost ease secur unaccep bundling online anonym budget {
		cap drop `CBDC_`ii''
		tempvar CBDC_`ii'
	}
	//change designs here
	if "`design'" == "design_d" {
		qui gen `CBDC_rate'     = rate
		qui gen `CBDC_cost'     = costopp_dc/sum_costopp           /*normalised by the overal perception*/
		qui gen `CBDC_ease'     = ease_dc/sum_ease
		qui gen `CBDC_secur'    = secur_dc/sum_secur
		qui gen `CBDC_unaccep'  = frac_card_unaccep
		qui gen `CBDC_bundling' = 1
		qui gen `CBDC_online' 	= 1
		qui gen `CBDC_anonym' 	= 0
		qui gen `CBDC_budget' 	= 0	
	}
	else if "`design'" == "design_c" {
		qui gen `CBDC_rate'     = 0
		qui gen `CBDC_cost'     = costopp_ca/sum_costopp
		qui gen `CBDC_ease'     = ease_ca/sum_ease
		qui gen `CBDC_secur'    = secur_ca/sum_secur
		qui gen `CBDC_unaccep'  = 0
		qui gen `CBDC_bundling' = 0
		qui gen `CBDC_online'   = 0
		qui gen `CBDC_anonym'   = 1
		qui gen `CBDC_budget'   = 1
	} 	
	else if "`design'" == "design_real" {                             /*baseline design of CBDC*/
		qui gen `CBDC_rate'     = 0
		qui gen `CBDC_cost'     = costopp_ca/sum_costopp
		qui gen `CBDC_ease'     = ease_ca/sum_ease
		qui gen `CBDC_secur'    = secur_ca/sum_secur
		qui gen `CBDC_unaccep'  = 0
		qui gen `CBDC_bundling' = 0
		qui gen `CBDC_online'   = 1
		qui gen `CBDC_anonym'   = 0.7               
		qui gen `CBDC_budget'   = 0.7
	} 	
	//change the values of one attribute while keeping others the same
	if "`charname'" == "d" | "`charname'" == "c" | "`charname'" == "real" {
		local charlist2   `charname'               /*where charname = d, c, or real*/
	}
	else {                     /*if charname = rate, for example*/
		if "`design'" == "design_d" {
			local baseline d 
		}
		else if "`design'" == "design_c" {
			local baseline c
		}
		else if "`design'" == "design_real" {
			local baseline real 
		}
		local charlist2 `baseline' `charname'
	}
	
	local iter = 0
	local char = 0	
	//for each feature such as rate, also need to run the baseline results to calculate the percentage changes from baseline, 
	//so construct the local charlist2 that includes the baseline case and one attribute
	foreach charname2 of local charlist2 { 
		local ++ char 
	
		forval jj = 1(1)`=colsof(mat_`charname2')' {
			local ++ iter
		
			forval yr = `yrmin'(1)`yrmax' {
				//columns: attr_baseline (attr_feature) rho gamma eta
				mat m_s_assets_`yr'[`iter', `char'] = mat_`charname2'[1, `jj']
				mat m_s_assets_`yr'[`iter', `= `char_num' + 1'] = `rho'
				mat m_s_assets_`yr'[`iter', `= `char_num' + 2'] = `gamma'
				mat m_s_assets_`yr'[`iter', `= `char_num' + 3'] = `eta'
	
			}
			forval kk = 1(1)`char_num' {
				if `kk' != `char' {
					forval yr = `yrmin'(1)`yrmax' {
						mat m_s_assets_`yr'[`iter', `kk'] = 9999               /*9999 refers to unchanged values*/
					}
				}
			}	
			//change attributes here
			if "`charname2'" == "d" | "`charname2'" == "c" | "`charname2'" == "real" {
				/*don't need to change anything*/
			}
			else if "`charname2'" == "cost_ease_secur" {
				local instr_index = 0
				//cc ca dc pc mo go onl onloth
				/*rating 1-cc 2-ca 3-dc 4-pc 5-mo*/
				foreach instr in cc ca dc pc mo {
					local ++ instr_index
					
					if `jj' == `instr_index' {
						di "jj `jj' = instr `instr'"
						cap drop `CBDC_cost'
						cap drop `CBDC_ease'
						cap drop `CBDC_secur'
						
						qui gen `CBDC_cost' = costopp_`instr'/sum_costopp
						qui gen `CBDC_ease' = ease_`instr'/sum_ease
						qui gen `CBDC_secur' = secur_`instr'/sum_secur
						
					}
				}  
			}
			else if "`charname2'" == "cost"  {
				local instr_index = 0
				//cc ca dc pc mo go onl onloth
				/*rating 1-cc 2-ca 3-dc 4-pc 5-mo*/
				foreach instr in cc ca dc pc mo {
					local ++ instr_index
					if `jj' == `instr_index' {
						di "jj `jj' = instr `instr'"
						cap drop `CBDC_cost'
						qui gen `CBDC_cost' = costopp_`instr'/sum_costopp
					}
				}  
			}
			else if "`charname2'" == "ease" | "`charname2'" == "secur" {
				local instr_index = 0
				//cc ca dc pc mo go onl onloth
				/*rating 1-cc 2-ca 3-dc 4-pc 5-mo*/
				foreach instr in cc ca dc pc mo {
					local ++ instr_index
					if `jj' == `instr_index' {
						di "jj `jj' = instr `instr'"
						cap drop `CBDC_`charname2''
						qui gen `CBDC_`charname2'' = `charname2'_`instr'/sum_`charname2'
					}
				}  
			}
			
			else {
				cap drop `CBDC_`charname2''
				qui gen `CBDC_`charname2'' = mat_`charname2'[1, `jj']             
			}
				
			*-----------------------------------------------------------------------------
			*CONSTRUCT UTILITIES FOR CBDC
			*-----------------------------------------------------------------------------
			cap drop dif_Vcbdc_Vd_`iter'
			cap drop dif_Vcbdc_Vc_`iter'	
			cap drop factor_`iter'
			cap drop s_cash_`iter'
			cap drop s_dep_`iter'
			cap drop s_CBDC_`iter'
			cap drop as_cash_`iter'
			cap drop as_dep_`iter'
			cap drop as_CBDC_`iter'
			cap drop d_cash_`iter'
			cap drop d_dep_`iter'
			cap drop dl_CBDC_`iter'
			cap drop dl_cash_`iter'
			cap drop dl_dep_`iter'
			cap drop dlp_CBDC_`iter'
			cap drop dlp_cash_`iter'
			cap drop dlp_dep_`iter'
		
			//Utility difference between CBDC and deposits: dif_Vcbdc_Vd = Vcbdc - Vd
			qui gen dif_Vcbdc_Vd_`iter'   =   b_rate*(`CBDC_rate' - rate)                      ///
										+ b_reldif_costopp_dc*(`CBDC_cost' - costopp_dc/sum_costopp)           ///
										+ b_reldif_ease_dc*(`CBDC_ease' - ease_dc/sum_ease)                   ///
										+ b_reldif_secur_dc*(`CBDC_secur' - secur_dc/sum_secur)                 ///
										+ b_frac_card_unaccep*(`CBDC_unaccep' - frac_card_unaccep)                                /// 
										+ b_bundling*bundling*(`CBDC_bundling' - 1)                  ///
										+ b_sn_loc_online_mop*sn_loc_online_mop*(`CBDC_online' - 1)         ///
										+ b_anonymI*(-anonymI)*(`CBDC_anonym' - 0)            ///        
										+ b_budgetI*(-budgetI)*(`CBDC_budget' - 0)   if sample == 1   
			//social demographic variables gamma_cbdc
			foreach demo of local demolist { 
				qui replace dif_Vcbdc_Vd_`iter' = dif_Vcbdc_Vd_`iter' + (`gamma' - 1)*b_`demo'*`demo'        if sample == 1      
			}
			//CBDC fixed effect eta_cbdc
			qui replace dif_Vcbdc_Vd_`iter' = dif_Vcbdc_Vd_`iter' + (`eta' - 1)*b_cons       if sample == 1  
			
			//Utility difference between CBDC and cash: dif_Vcbdc_Vc = Vcbdc - Vc
			qui gen	dif_Vcbdc_Vc_`iter' = 	  b_rate*(`CBDC_rate' - 0)                      ///
										+ b_reldif_costopp_dc*(`CBDC_cost' - costopp_ca/sum_costopp)           ///
										+ b_reldif_ease_dc*(`CBDC_ease' - ease_ca/sum_ease)                   ///
										+ b_reldif_secur_dc*(`CBDC_secur' - secur_ca/sum_secur)                 ///
										+ b_frac_card_unaccep*(`CBDC_unaccep' - 0)                    /// 
										+ b_bundling*bundling*(`CBDC_bundling' - 0)                   ///
										+ b_sn_loc_online_mop*sn_loc_online_mop*(`CBDC_online' - 0)         ///
										+ b_anonymI*(-anonymI)*(`CBDC_anonym' - 1)            ///        
										+ b_budgetI*(-budgetI)*(`CBDC_budget' - 1)        if sample == 1	 	 							
			//social demographic variables gamma_cbdc
			foreach demo of local demolist { 
				qui replace dif_Vcbdc_Vc_`iter' = dif_Vcbdc_Vc_`iter' + (`gamma' - 0)*b_`demo'*`demo'    if sample == 1   
			}
			//CBDC fixed effect eta_cbdc
			qui replace dif_Vcbdc_Vc_`iter' = dif_Vcbdc_Vc_`iter' + (`eta' - 0)*b_cons       if sample == 1   	
		
			
			//under nested logit model, if CBDC and deposit are in the same nest
			//nested logit - when tau approaches 0, rho is 1; when tau is 1, rho is 0
			//besides, V_cbdc - V_d is scaled by 1/tau
			if "`model'" ==  "nest_d_cbdc" {
			
				local tau_d = `tau'
				
				//factor = (1 + s_cbdc/s_d)^(1 - tau_d)
				//sc'/sd' = factor*sc/sd
				//s_cbdc/s_d = exp(dif_Vcbdc_Vd_`iter'/`tau_d')
				qui gen double factor_`iter' = (1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(1 - `tau_d') 
				//household-level shares
				qui gen double s_dep_`iter'  = 1/(1 + factor_`iter'*exp(- xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))
				qui gen double s_CBDC_`iter' = exp(dif_Vcbdc_Vd_`iter'/`tau_d')*s_dep_`iter'
				qui gen double s_cash_`iter' = 1 - s_dep_`iter' - s_CBDC_`iter'
				
			}
			
			//under nested logit model, if CBDC and cash are in the same nest
			if "`model'" ==  "nest_c_cbdc" {
			
				local tau_c = `tau'
				
				//factor = (1 + s_cbdc/s_c)^(1 - tau_c)
				//sd'/sc' = factor*sd/sc
				//s_cbdc/s_c = exp(dif_Vcbdc_Vc_`iter'/`tau_c')
				qui gen double factor_`iter' = (1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(1 - `tau_c') 
				//household-level shares
				qui gen double s_cash_`iter' = 1/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))
				qui gen double s_CBDC_`iter' = exp(dif_Vcbdc_Vc_`iter'/`tau_c')*s_cash_`iter'
				qui gen double s_dep_`iter'  = 1 - s_cash_`iter' - s_CBDC_`iter'
			}
			
			//first step to compute aggregate shares (unweighted by sample weights but weighted by wealth weight) before summing across households
			qui gen double as_cash_`iter' = s_cash_`iter'*wealth_frac 	
			qui gen double as_CBDC_`iter' = s_CBDC_`iter'*wealth_frac  	
			qui gen double as_dep_`iter'  = s_dep_`iter'*wealth_frac  		 
			
			//changes in cash or deposit in percentage terms
			qui gen double d_cash_`iter' = (s_cash_`iter' - s_cash_old)/s_cash_old*100    
			qui gen double d_dep_`iter'  = (s_dep_`iter' - s_dep_old)/s_dep_old*100 	
			
			//percentage point changes in aggregate shares relative to the base 
			//the first row is the baseline result
			local base = 1
			di "base = `base'"
			
			qui gen double dl_CBDC_`iter' = as_CBDC_`iter' - as_CBDC_`base'           /*iter1 = baseline design*/
			qui gen double dl_cash_`iter' = as_cash_`iter' - as_cash_`base' 
			qui gen double dl_dep_`iter'  = as_dep_`iter'  - as_dep_`base'	
		
			*-----------------------------------------------------------------------------
			*USE DELTA METHOD TO FIND STANDARD ERROR OF DIFFERENT ESTIMATES
			*-----------------------------------------------------------------------------
			local num = 1
			//num refers to each coefficient in sequence of the covariance matrix 
			//differentiate predicted variables (e.g. CBDC share, deposit share, etc) wrt each parameter b
			foreach x of local xlist {
				cap drop z`num'_`iter'  
				cap drop db`num'_d_dep
				cap drop db`num'_d_cash
				cap drop db`num'_dl_CBDC
				cap drop db`num'_dl_dep
				cap drop db`num'_dl_cash
				cap drop db`num'_as_CBDC
				cap drop db`num'_as_dep
				cap drop db`num'_as_cash
				cap drop db`num'_as_CBDC_base
				cap drop db`num'_as_dep_base
				cap drop db`num'_as_cash_base								
				
				////////////////////////////////////////////////////////////////////////	
				//z`num' represents the difference in attribute between CBDC and deposit (cash) when CBDC and deposit (cash) are in one nest. 
				if "`x'" == "rate" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_rate' - rate
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_rate' - 0
					}
				}
				else if "`x'" == "reldif_costopp_dc" {	
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_cost' - costopp_dc/sum_costopp
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_cost' - costopp_ca/sum_costopp
					}
				}
				else if "`x'" == "reldif_ease_dc" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_ease' - ease_dc/sum_ease
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_ease' - ease_ca/sum_ease
					}
				}
				else if "`x'" == "reldif_secur_dc" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_secur' - secur_dc/sum_secur  
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_secur' - secur_ca/sum_secur  
					}
				}
				else if "`x'" == "frac_card_unaccep" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_unaccep' - frac_card_unaccep              
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  `CBDC_unaccep' - 0
					}
				}
				else if "`x'" == "bundling" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_bundling' - 1)*bundling
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_bundling' - 0)*bundling
					}
				}
				else if "`x'" == "sn_loc_online_mop" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_online' - 1)*sn_loc_online_mop
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_online' - 0)*sn_loc_online_mop
					}
				}
				else if "`x'" == "anonymI" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_anonym' - 0)*(-anonymI)
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_anonym' - 1)*(-anonymI)
					}
				}
				else if "`x'" == "budgetI" {
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_budget' - 0)*(-budgetI)
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`CBDC_budget' - 1)*(-budgetI)
					}
				}
				else if "`x'" == "cons"  {	
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`eta' - 1)
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`eta' - 0)
					}
				}
				//for the demographic variables, the differences between CBDC and deposit/cash are in terms of the parameters
				else {	
					if "`model'" ==  "nest_d_cbdc" {
						qui gen z`num'_`iter' =  (`gamma' - 1)*`x'                /*social demo vars and year dummies*/
					}
					else if "`model'" ==  "nest_c_cbdc" {
						qui gen z`num'_`iter' =  (`gamma' - 0)*`x'  
					}
				}
		
				
				////////////////////////////////////////////////////////////////////////	
				//under nested logit, assume CBDC and deposit are in one nest
				if "`model'" ==  "nest_d_cbdc" {
					////////////shares 
					//derivative of each household's CBDC share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_CBDC = (exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - exp(dif_Vcbdc_Vd_`iter'/`tau_d')*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's new cash share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_cash = (((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - factor_`iter'*exp(-xb)*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's new deposit share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_dep = (- ((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					
					///////////percentage change in deposit or cash
					//derivative of each household's change in deposit relative to deposit before CBDC issuance wrt the coefficient b_`num' multiplied by 100 to get percentage terms
					qui gen double db`num'_d_dep = (exp(-xb)*(-`x')/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - (1 + exp(-xb))*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)*100  if sample == 1 
					
					//derivative of each household's change in cash relative to cash before CBDC issuance wrt the coefficient b_`num' multiplied by 100 to get percentage terms
					qui gen double db`num'_d_cash = (((1 + exp(-xb))*(1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d') + factor_`iter'*exp(-xb)*(-`x'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - factor_`iter'*(1 + exp(-xb))*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)*100 if sample == 1 	
					
					//////percentage point change in level  - not used in paper
					qui gen double db`num'_dl_CBDC = ((exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - exp(dif_Vcbdc_Vd_`iter'/`tau_d')*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2) - (exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d')) - exp(dif_Vcbdc_Vd_`base'/`tau_d')*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2))*wealth_frac if sample == 1 
					
					
					qui gen double db`num'_dl_cash = ((((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d')) - factor_`iter'*exp(-xb)*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2)  -  (((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d')) - factor_`base'*exp(-xb)*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2))*wealth_frac if sample == 1 
					
													
					qui gen double db`num'_dl_dep =  ((- ((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d')*exp(-xb) + factor_`iter'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`iter'/`tau_d')*(z`num'_`iter'/`tau_d'))/(1 + factor_`iter'*exp(-xb) + exp(dif_Vcbdc_Vd_`iter'/`tau_d'))^2) - (- ((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2))*wealth_frac if sample == 1 
					
					//////percentage change in level
					//derivative of each household's CBDC share under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_CBDC above except that `iter' is replaced by `base'
					qui gen double db`num'_as_CBDC_base = (exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d')) - exp(dif_Vcbdc_Vd_`base'/`tau_d')*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's cash share under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_cash above except that `iter' is replaced by `base'
					qui gen double db`num'_as_cash_base = (((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d')) - factor_`base'*exp(-xb)*((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's CBDC deposit under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_dep above except that `iter' is replaced by `base'
					qui gen double db`num'_as_dep_base = (- ((1-`tau_d')*(1 + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^(-`tau_d')*exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d')*exp(-xb) + factor_`base'*exp(-xb)*(-`x') + exp(dif_Vcbdc_Vd_`base'/`tau_d')*(z`num'_`base'/`tau_d'))/(1 + factor_`base'*exp(-xb) + exp(dif_Vcbdc_Vd_`base'/`tau_d'))^2)*wealth_frac if sample == 1 
					
					
					
				}
			
				//under nested logit, assume CBDC and cash are in one nest
				if "`model'" ==  "nest_c_cbdc" {
					
					////////////shares 
					//derivative of each household's CBDC share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_CBDC = (exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c')) - exp(dif_Vcbdc_Vc_`iter'/`tau_c')*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's new deposit share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_dep = (((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c')) - factor_`iter'*exp(xb)*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's new cash share wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					qui gen double db`num'_as_cash = (- ((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)*wealth_frac if sample == 1 
											
											
					///////////percentage change in deposit or cash 
					//derivative of each household's change in deposit relative to deposit before CBDC issuance wrt the coefficient b_`num' multiplied by 100 to get percentage terms
					qui gen double db`num'_d_dep = (((1 + exp(xb))*(1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c') + factor_`iter'*exp(xb)*(`x'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c')) - factor_`iter'*(1 + exp(xb))*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)*100 if sample == 1 
					
					//derivative of each household's change in cash relative to cash before CBDC issuance wrt the coefficient b_`num' multiplied by 100 to get percentage terms
					qui gen double db`num'_d_cash = (exp(xb)*(`x')/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))  - (1 + exp(xb))*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)*100    if sample == 1 
					
					
					////////////percentage point change in level  - not used in paper
					qui gen double db`num'_dl_CBDC = ((exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c')) - exp(dif_Vcbdc_Vc_`iter'/`tau_c')*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2)- (exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c')) - exp(dif_Vcbdc_Vc_`base'/`tau_c')*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2))*wealth_frac if sample == 1 
					
					qui gen double db`num'_dl_dep =  ((((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c')) - factor_`iter'*exp(xb)*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2) - (((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c')) - factor_`base'*exp(xb)*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2) )*wealth_frac if sample == 1 
					
					qui gen double db`num'_dl_cash = ((- ((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c')*exp(xb) + factor_`iter'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`iter'/`tau_c')*(z`num'_`iter'/`tau_c'))/(1 + factor_`iter'*exp(xb) + exp(dif_Vcbdc_Vc_`iter'/`tau_c'))^2) -  (- ((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2) )*wealth_frac if sample == 1 
					
					
					//////////percentage change in level   
					//derivative of each household's CBDC share under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_CBDC above except that `iter' is replaced by `base'
					qui gen double db`num'_as_CBDC_base = (exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c')) - exp(dif_Vcbdc_Vc_`base'/`tau_c')*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's deposit share under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_dep above except that `iter' is replaced by `base'
					qui gen double db`num'_as_dep_base = (((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c')) - factor_`base'*exp(xb)*((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2)*wealth_frac if sample == 1 
					
					//derivative of each household's cash share under baseline design wrt each coefficient b_`num' multiplied by wealth fraction w_i/w
					//same as db`num'_as_cash above except that `iter' is replaced by `base'
					qui gen double db`num'_as_cash_base = (- ((1-`tau_c')*(1 + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^(-`tau_c')*exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c')*exp(xb) + factor_`base'*exp(xb)*(`x') + exp(dif_Vcbdc_Vc_`base'/`tau_c')*(z`num'_`base'/`tau_c'))/(1 + factor_`base'*exp(xb) + exp(dif_Vcbdc_Vc_`base'/`tau_c'))^2)*wealth_frac if sample == 1 
		
				}
		
				local ++num 
			}
			
			*-------------------------------------------------------------------
			//compute standard errors of the predicted values for each year using the household-level derivatives above
			forval yr = `yrmin'(1)`yrmax' {              /*predict CBDC for each year*/
				di "year = `yr'"
				//standard errors for aggregate deposit share, cash share, CBDC share, percentage point change in levels of deposit, cash and CBDC.
				foreach sv in as_dep as_cash as_CBDC  dl_dep dl_cash dl_CBDC {
					//generate empty gradient vector G
					mat G_`sv' = J(`=colsof(V)' , 1, .)
					//foreach k-th parameter b_`num', calculate the derivative of the aggregate variable with respect to that parameter
					forval num = 1(1)`=colsof(V)' {                        /*for each parameter num*/
						qui su db`num'_`sv'        if year == `yr'
						mat G_`sv'[`num', 1] = `r(mean)'*`r(N)'                /*sum over the values = mean*number of observations  to get the derivative of the aggregate values wrt a parameter*/
					}
					//Variance = G'VG
					mat GVG_`sv' = G_`sv''*V*G_`sv'
					scalar se_`sv' = sqrt(el(GVG_`sv', 1, 1))
					scalar list se_`sv'
				}
				//standard errors for dlp*  - percentage change in aggregate shares relative to baseline shares
				foreach sv in dep cash CBDC {
					//note that as_`sv'_`base' is already multiplied by the wealth weight
					qui su as_`sv'_`base'       if year == `yr'
					scalar sum_as_`sv'_`base' = `r(mean)'*`r(N)'             /*summing across households in a given year to get aggregate baseline share*/
					qui su as_`sv'_`iter'       if year == `yr'
					scalar sum_as_`sv'_`iter' = `r(mean)'*`r(N)'             /*summing across households in a given year to get aggregate share*/
					
					mat G_dlp_`sv' = J(`=colsof(V)' , 1, .)
					forval num = 1(1)`=colsof(V)' {
						qui su db`num'_as_`sv'        if year == `yr'
						scalar sum_db`num'_as_`sv' = `r(mean)'*`r(N)' 
						qui su db`num'_as_`sv'_base        if year == `yr'
						scalar sum_db`num'_as_`sv'_base = `r(mean)'*`r(N)' 
						//derivative of percentage change in aggregate share relative to baseline share wrt the k-th parameter b_`num'
						// *100 to reflect percentage terms
						mat G_dlp_`sv'[`num', 1] =   (1/sum_as_`sv'_`base'*sum_db`num'_as_`sv' -   sum_as_`sv'_`iter'/(sum_as_`sv'_`base'^2)*sum_db`num'_as_`sv'_base )*100         
					}
					mat GVG_dlp_`sv' = G_dlp_`sv''*V*G_dlp_`sv'
					scalar se_dlp_`sv' = sqrt(el(GVG_dlp_`sv', 1, 1))
					scalar list se_dlp_`sv'
		
				}
				//standard errors for mean/median percentage change in deposit or cash across households
				foreach sv in d_dep d_cash {
					//empty gradient vector for derivatives of mean percentage change in deposit or cash wrt each parameter
					mat G_m_`sv' = J(`=colsof(V)' , 1, .)
					//empty gradient vector for derivatives of median percentage change in deposit or cash wrt each parameter
					mat G_p50_`sv' = J(`=colsof(V)' , 1, .)
					
					forval num = 1(1)`=colsof(V)' {
						qui su db`num'_`sv'       if year == `yr', d
						mat G_m_`sv'[`num', 1] = `r(mean)'
						mat G_p50_`sv'[`num', 1] = `r(p50)'
					}
					mat GVG_m_`sv' = G_m_`sv''*V*G_m_`sv'
					mat GVG_p50_`sv' = G_p50_`sv''*V*G_p50_`sv'
					scalar se_m_`sv' = sqrt(el(GVG_m_`sv', 1, 1))
					scalar se_p50_`sv' = sqrt(el(GVG_p50_`sv', 1, 1))
				}
				
				*-------------------------------------------------------------------
				//save the predicted values in matrices	for each year 
				//note that the order of the following loops and variables are consistent with the column names defined above
				local index = 4
				//save the mean cash share, deposit share, CBDC share 
				foreach stat in s_cash s_dep s_CBDC {
					qui su `stat'_`iter'       if year == `yr'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `r(mean)'
					local ++ index
				}	
				//generate and save the aggregate cash share, deposit share, CBDC share, and percentage point change in cash, deposit and CBDC 
				foreach stat in as_cash as_dep as_CBDC dl_cash dl_dep dl_CBDC {
					qui su `stat'_`iter'       if year == `yr'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `r(mean)'*`r(N)'
					local ++ index
				}	
				//generate and save percentage change in cash, deposit and CBDC relative to baseline design
				foreach stat in cash dep CBDC {
					scalar dlp_`stat'_`iter' = (sum_as_`stat'_`iter'/sum_as_`stat'_`base' - 1 )*100            /*sum_* variables are already conditioned on year, percentage changes*/
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = dlp_`stat'_`iter'
					local ++ index
				}
				//save the mean and median percentage change in cash or deposit relative to their values before CBDC issuance
				foreach stat in d_cash d_dep  {	
					qui su `stat'_`iter'       if year == `yr', d
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `r(mean)'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 1'] = `r(p50)'
					local index = `index' + 2
				}
				//generate and save confidence intervals for aggregate shares and percentage point change in shares 
				foreach stat in as_cash as_dep as_CBDC dl_cash dl_dep dl_CBDC  {
					qui su `stat'_`iter'       if year == `yr'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `r(mean)'*`r(N)' - 1.96*se_`stat'                /*aggregate stats = `r(mean)'*`r(N)'*/
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 1'] = `r(mean)'*`r(N)' + 1.96*se_`stat'
					local index = `index' + 2
				}
				//generate and save confidence intervals for percentage change in aggregate shares relative to shares under baseline design
				foreach stat in dlp_cash dlp_dep dlp_CBDC {                   /*these 3 stats are scalars*/
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `stat'_`iter' - 1.96*se_`stat'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 1'] = `stat'_`iter' + 1.96*se_`stat'
					local index = `index' + 2
				}
				//generate and save confidence intervals for mean/median percentage change in cash or deposit 
				foreach stat in d_cash d_dep  {
					qui su `stat'_`iter'       if year == `yr', d
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index''] = `r(mean)' - 1.96*se_m_`stat'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 1'] = `r(mean)' + 1.96*se_m_`stat'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 2'] = `r(p50)' - 1.96*se_p50_`stat'
					mat m_s_assets_`yr'[`iter', `= `char_num' + `index' + 3'] = `r(p50)' + 1.96*se_p50_`stat'
					
					local index = `index' + 4
				}
			}   
		}
	}
	
	//save the matrix for each year in separate data files
	forval yr = `yrmin'(1)`yrmax' {
		mat list m_s_assets_`yr'
		local colnamelist `attr_list' `columnvars'
	
		foreach v of local colnamelist {
			cap drop `v'
		}
		svmat m_s_assets_`yr', names(col)
		//save the matrices in dta files
		preserve
		local firstword : word 1 of `attr_list'
		keep `firstword'-d_dep_median_hi
		drop if attr_`charname' == .     
		gen nest = "`model'"
		gen design = "`design'"
		order nest design
		
		save "$data\\counterfactual analysis\\$date\\no bank responses\\meanshares_dshares_`model'_`design'_`charname'_rho`rho'_gamma`gamma'_eta`eta'_year`yr'.dta", replace

		restore
	}
	

end



********************************************************************************
*Run different specifications in parallel
********************************************************************************


local rho_list      0    0.2    0.4    0.5   0.6   0.8   0.99    0.999
local gamma_list    0    0.2    0.4    0.5   0.6   0.8   1
local eta_list      0    0.2    0.4    0.5   0.6   0.8   1


//Set the number of cores to be used 
global nclusters = 12
parallel initialize $nclusters


local joblist = ""
foreach model in nest_d_cbdc  nest_c_cbdc  {
	
	foreach design in  design_real  design_d  design_c   {
		if "`design'" == "design_d" {
			local charlist d 
		}
		else if "`design'" == "design_c" {
			local charlist c
		}
		else if "`design'" == "design_real" {
			local charlist real 
		}
		//CHANGE HERE!
		//If only study the cbdc demand under basic designs -> local charlist `charlist' 
		local featurelist  rate  bundling  online  anonym  budget  ease  secur  cost unaccep  cost_ease_secur                    
		local charlist `charlist'  `featurelist'
		
		foreach charname of local charlist {
		
			foreach rho of local rho_list {
			
				foreach gamma of local gamma_list  {
					
					//if not studying attribute change
					if  "`featurelist'" == "" {
						foreach eta of local eta_list {
							local joblist `joblist' `model'XXX`design'XXX`charname'XXX`rho'XXX`gamma'XXX`eta'
						}
					}
					//when studying the attributes, set eta and gamma the same
					if  "`featurelist'" != "" {
						local eta  `gamma'       
						local joblist `joblist' `model'XXX`design'XXX`charname'XXX`rho'XXX`gamma'XXX`eta'
					}
				
				}
			}
		}	
	}
}
di "`joblist'"

//Note that parfor.ado is written separately
parallel, prog(prediction) nodata: parfor, nclusters($nclusters)  program("prediction") joblist("`joblist'")                               




					
						
				
						
						
						
						
						
						